package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.HttpUtils;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouSupplyChainService;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class settlementJob {


    @Autowired
    private IKuaishouSupplyChainService ikuaishouSupplyChainService;


    public static void main(String[] args) {
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String startDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -31);
        System.err.println(startDate);
    }

    @XxlJob("refreshCourierNumber")
    public void refreshCourierNumber() throws Exception {
        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        String startDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -31);
        List<JSONObject> list = ikuaishouSupplyChainService.getSamplesData(startDate);
        if (Check.isNull(list)) {
            return;
        }
        for (JSONObject jsonObject : list) {
            Long id = jsonObject.getLong("id");
            String courierNumber = jsonObject.getString("courierNumber");
            if (Check.isNull(courierNumber)) {
                continue;
            }
            String companyCode = jsonObject.getString("companyCode");
            if (Check.isNull(companyCode)) {
                continue;
            }
            Map<String, Object> map = new HashMap<>();
            map.put("id", id);
            map.put("courierNumber", courierNumber);
            map.put("companyCode", companyCode);
            HttpUtils.httpGet("http://ruixuan.api.tjyourong.com.cn/itemCollectSamples/refreshCourierNumber", map, null);
            Thread.sleep(1000L);
        }


    }


    /**
     * 自定义创建监测链接预警
     */
    @XxlJob("cleanSettlementData")
    public void cleanSettlementData() {
        String url = "http://ruixuan.api.tjyourong.com.cn/settlement/data";
        String date = DateUtils.getLastDay(DateUtils.date2Str(), 1);
        Long statDate = Long.valueOf(date.replace("-", ""));
        List<JSONObject> list = ikuaishouSupplyChainService.getDataList(statDate);
        if (Check.isNull(list)) {
            return;
        }
        List<List<JSONObject>> partition = Lists.partition(list, 1000);
        for (int i = 0; i < partition.size(); i++) {
            List<JSONObject> jsonObjects = partition.get(i);
            Map<String, Object> param = new HashMap<>();
            param.put("statDate", statDate);
            param.put("data", jsonObjects);
            HttpUtils.httpPostRequest(url, param, null);
        }

    }


    @XxlJob("cleanPromoterSettlementData")
    public void cleanPromoterSettlementData() {
        String url = "http://ruixuan.api.tjyourong.com.cn/settlement/promoterData";
        String date = DateUtils.getLastDay(DateUtils.date2Str(), 1);
        Long statDate = Long.valueOf(date.replace("-", ""));
        List<JSONObject> list = ikuaishouSupplyChainService.getPromoterDataList(statDate);
        if (Check.isNull(list)) {
            return;
        }
        List<List<JSONObject>> partition = Lists.partition(list, 1000);
        for (int i = 0; i < partition.size(); i++) {
            List<JSONObject> jsonObjects = partition.get(i);
            Map<String, Object> param = new HashMap<>();
            param.put("statDate", statDate);
            param.put("data", jsonObjects);
            HttpUtils.httpPostRequest(url, param, null);
        }

    }


    @XxlJob("cleanPromoterBindChannel")
    public void cleanPromoterBindChannel() {
        Long date = Long.valueOf(DateUtils.getNowDate("yyyyMMdd"));
        ikuaishouSupplyChainService.deletePromoterBindChannelByDate(date);
        ikuaishouSupplyChainService.cleanKuaiShouPromoterBindChannel(date);
        ikuaishouSupplyChainService.cleanBytedancePromoterBindChannel(date);

    }


}
