package cn.com.ctop.kuaishou.modules.dimension.controller;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.dimension.entity.KuaiShouDimensionMaterial;
import cn.com.ctop.kuaishou.modules.dimension.service.IKuaiShouDimensionMaterialService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 多维度测试素材列表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-07-23
 */
@Slf4j
@Api(tags = "多维度测试素材列表")
@RestController
@RequestMapping("/dimension/kuaiShouDimensionMaterial")
public class KuaiShouDimensionMaterialController {
    @Autowired
    private IKuaiShouDimensionMaterialService kuaiShouDimensionMaterialService;


    @ApiOperation(value = "多维度测试素材列表-添加", notes = "多维度测试素材列表-添加")
    @PostMapping(value = "/getVideoList")
    public Result<PageInfo<List<JSONObject>>> getVideoList(@RequestBody JSONObject requestJson) {
        Result<PageInfo<List<JSONObject>>> result = new Result<>();
        try {
            Long id = requestJson.getLong("id");
            if (Check.isNull(id)) {
                throw new Exception("策略id不能为空");
            }
            Long accountId = requestJson.getLong("accountId");
            if (Check.isNull(accountId)) {
                throw new Exception("账户id不能为空");
            }
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("strategyId", id);
            requestMap.put("accountId", accountId);

            String startDate = requestJson.getString("startDate");
            String endDate = requestJson.getString("endDate");
            if (!Check.isNull(startDate) && !Check.isNull(endDate)) {
                requestMap.put("startDate", startDate + " 00:00:01");
                requestMap.put("endDate", endDate + " 23:59:59");
            }
            Integer pageNo = requestJson.getInteger("pageNo");
            Integer pageSize = requestJson.getInteger("pageSize");
            PageInfo<List<JSONObject>> videoList = kuaiShouDimensionMaterialService.getVideoList(requestMap, pageNo, pageSize);
            result.success("查询成功");
            result.setResult(videoList);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("查询失败");
        }
        return result;
    }


    /**
     * 分页列表查询
     *
     * @param kuaiShouDimensionMaterial
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "多维度测试素材列表-分页列表查询", notes = "多维度测试素材列表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouDimensionMaterial>> queryPageList(KuaiShouDimensionMaterial kuaiShouDimensionMaterial,
                                                                  @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                  HttpServletRequest req) {
        Result<IPage<KuaiShouDimensionMaterial>> result = new Result<>();
        QueryWrapper<KuaiShouDimensionMaterial> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouDimensionMaterial, req.getParameterMap());
        Page<KuaiShouDimensionMaterial> page = new Page<KuaiShouDimensionMaterial>(pageNo, pageSize);
        IPage<KuaiShouDimensionMaterial> pageList = kuaiShouDimensionMaterialService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouDimensionMaterial
     * @return
     */
    @ApiOperation(value = "多维度测试素材列表-添加", notes = "多维度测试素材列表-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouDimensionMaterial> add(@RequestBody KuaiShouDimensionMaterial kuaiShouDimensionMaterial) {
        Result<KuaiShouDimensionMaterial> result = new Result<>();
        try {
            kuaiShouDimensionMaterialService.save(kuaiShouDimensionMaterial);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouDimensionMaterial
     * @return
     */
    @ApiOperation(value = "多维度测试素材列表-编辑", notes = "多维度测试素材列表-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaiShouDimensionMaterial> edit(@RequestBody KuaiShouDimensionMaterial kuaiShouDimensionMaterial) {
        Result<KuaiShouDimensionMaterial> result = new Result<KuaiShouDimensionMaterial>();
        KuaiShouDimensionMaterial kuaiShouDimensionMaterialEntity = kuaiShouDimensionMaterialService.getById(kuaiShouDimensionMaterial.getId());
        if (kuaiShouDimensionMaterialEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouDimensionMaterialService.updateById(kuaiShouDimensionMaterial);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "多维度测试素材列表-通过id删除", notes = "多维度测试素材列表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            kuaiShouDimensionMaterialService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "多维度测试素材列表-批量删除", notes = "多维度测试素材列表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouDimensionMaterial> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<KuaiShouDimensionMaterial> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouDimensionMaterialService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "多维度测试素材列表-通过id查询", notes = "多维度测试素材列表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouDimensionMaterial> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouDimensionMaterial> result = new Result<>();
        KuaiShouDimensionMaterial kuaiShouDimensionMaterial = kuaiShouDimensionMaterialService.getById(id);
        if (kuaiShouDimensionMaterial == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouDimensionMaterial);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaiShouDimensionMaterial> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaiShouDimensionMaterial kuaiShouDimensionMaterial = JSON.parseObject(deString, KuaiShouDimensionMaterial.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouDimensionMaterial, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaiShouDimensionMaterial> pageList = kuaiShouDimensionMaterialService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "多维度测试素材列表列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaiShouDimensionMaterial.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("多维度测试素材列表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaiShouDimensionMaterial> listKuaiShouDimensionMaterials = ExcelImportUtil.importExcel(file.getInputStream(), KuaiShouDimensionMaterial.class, params);
                kuaiShouDimensionMaterialService.saveBatch(listKuaiShouDimensionMaterials);
                return Result.ok("文件导入成功！数据行数:" + listKuaiShouDimensionMaterials.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
