package cn.com.ctop.kuaishou.modules.dimension.controller;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.dimension.entity.KuaiShouDimensionVariable;
import cn.com.ctop.kuaishou.modules.dimension.service.IKuaiShouDimensionVariableService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 多维度测试变量配置
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-07-20
 */
@Slf4j
@Api(tags = "多维度测试变量配置")
@RestController
@RequestMapping("/dimension/kuaiShouDimensionVariable")
public class KuaiShouDimensionVariableController {
    @Autowired
    private IKuaiShouDimensionVariableService kuaiShouDimensionVariableService;

    /**
     * 分页列表查询
     *
     * @param kuaiShouDimensionVariable
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "多维度测试变量配置-分页列表查询", notes = "多维度测试变量配置-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouDimensionVariable>> queryPageList(KuaiShouDimensionVariable kuaiShouDimensionVariable,
                                                                  @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                  HttpServletRequest req) {
        Result<IPage<KuaiShouDimensionVariable>> result = new Result<>();
        QueryWrapper<KuaiShouDimensionVariable> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouDimensionVariable, req.getParameterMap());
        Page<KuaiShouDimensionVariable> page = new Page<KuaiShouDimensionVariable>(pageNo, pageSize);
        IPage<KuaiShouDimensionVariable> pageList = kuaiShouDimensionVariableService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


    @ApiOperation(value = "多维度测试变量配置-分页列表查询", notes = "多维度测试变量配置-分页列表查询")
    @GetMapping(value = "/selectVariableByStrategyId")
    public Result<KuaiShouDimensionVariable> selectVariableByStrategyId(Long strategyId) {
        Result<KuaiShouDimensionVariable> result = new Result<>();
        QueryWrapper<KuaiShouDimensionVariable> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("strategy_id", strategyId);
        KuaiShouDimensionVariable one = kuaiShouDimensionVariableService.getOne(queryWrapper);
        result.setSuccess(true);
        result.setResult(one);
        return result;
    }


    /**
     * 添加
     *
     * @param kuaiShouDimensionVariable
     * @return
     */
    @ApiOperation(value = "多维度测试变量配置-添加", notes = "多维度测试变量配置-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouDimensionVariable> add(@RequestBody KuaiShouDimensionVariable kuaiShouDimensionVariable) {
        Result<KuaiShouDimensionVariable> result = new Result<>();
        try {
            Long strategyId = kuaiShouDimensionVariable.getStrategyId();
            if (Check.isNull(strategyId)) {
                throw new Exception("策略id不能为空");
            }
            QueryWrapper<KuaiShouDimensionVariable> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("strategy_id", strategyId);
            queryWrapper.last("limit 1");
            KuaiShouDimensionVariable one = kuaiShouDimensionVariableService.getOne(queryWrapper);
            if (!Check.isNull(one)) {
                throw new Exception("该策略已绑定变量模板");
            }
            kuaiShouDimensionVariableService.save(kuaiShouDimensionVariable);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouDimensionVariable
     * @return
     */
    @ApiOperation(value = "多维度测试变量配置-编辑", notes = "多维度测试变量配置-编辑")
    @PostMapping(value = "/edit")
    public Result<KuaiShouDimensionVariable> edit(@RequestBody KuaiShouDimensionVariable kuaiShouDimensionVariable) {
        Result<KuaiShouDimensionVariable> result = new Result<KuaiShouDimensionVariable>();
        KuaiShouDimensionVariable kuaiShouDimensionVariableEntity = kuaiShouDimensionVariableService.getById(kuaiShouDimensionVariable.getId());
        if (kuaiShouDimensionVariableEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouDimensionVariableService.updateById(kuaiShouDimensionVariable);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "多维度测试变量配置-通过id删除", notes = "多维度测试变量配置-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            kuaiShouDimensionVariableService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "多维度测试变量配置-批量删除", notes = "多维度测试变量配置-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouDimensionVariable> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<KuaiShouDimensionVariable> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouDimensionVariableService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "多维度测试变量配置-通过id查询", notes = "多维度测试变量配置-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouDimensionVariable> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouDimensionVariable> result = new Result<>();
        KuaiShouDimensionVariable kuaiShouDimensionVariable = kuaiShouDimensionVariableService.getById(id);
        if (kuaiShouDimensionVariable == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouDimensionVariable);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "多维度测试变量配置-通过id查询", notes = "多维度测试变量配置-通过id查询")
    @GetMapping(value = "/queryByStrategyId")
    public Result<KuaiShouDimensionVariable> queryByStrategyId(Long strategyId) {
        Result<KuaiShouDimensionVariable> result = new Result<>();
        QueryWrapper<KuaiShouDimensionVariable> variableQueryWrapper = new QueryWrapper<>();
        variableQueryWrapper.eq("strategy_id", strategyId);
        variableQueryWrapper.last("limit 1");
        KuaiShouDimensionVariable one = kuaiShouDimensionVariableService.getOne(variableQueryWrapper);
        //   KuaiShouDimensionVariable kuaiShouDimensionVariable = kuaiShouDimensionVariableService.getById(id);
        if (one == null) {
            result.error500("未找到对应实体");
            result.setResult(null);
        } else {
            result.setResult(one);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaiShouDimensionVariable> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaiShouDimensionVariable kuaiShouDimensionVariable = JSON.parseObject(deString, KuaiShouDimensionVariable.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouDimensionVariable, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaiShouDimensionVariable> pageList = kuaiShouDimensionVariableService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "多维度测试变量配置列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaiShouDimensionVariable.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("多维度测试变量配置列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaiShouDimensionVariable> listKuaiShouDimensionVariables = ExcelImportUtil.importExcel(file.getInputStream(), KuaiShouDimensionVariable.class, params);
                kuaiShouDimensionVariableService.saveBatch(listKuaiShouDimensionVariables);
                return Result.ok("文件导入成功！数据行数:" + listKuaiShouDimensionVariables.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
